/*
  Copyright (C) 2002-2020 CERN for the benefit of the ATLAS collaboration
*/

#ifndef SwitchingFieldManager_H
#define SwitchingFieldManager_H

/** @class SwitchingFieldManager SwitchingFieldManager.h
 *
 *  @brief G4FieldManager that sets tight stepping for muons; disables
 *   magnetic field for other particles
 *
 *  @authors  Evangelos Kourlitis, John Apostolakis
 *  @date   2020-10-29
 */

#include "G4FieldManager.hh"

class SwitchingFieldManager : public G4FieldManager
{
  public:
    // using G4FieldManager::G4FieldManager;
    
    // Switch the field off when inside Calo
    SwitchingFieldManager(G4Field *field);
   
    ~SwitchingFieldManager();

    // Necessary virtual methods
    void   ConfigureForTrack( const G4Track * ) override;
    G4FieldManager* Clone() const override;

  private:
    G4Field*  m_savedField = nullptr;
    // Calo barrel region 1500mm < r < 3700mm and |z|<1000mm
    G4double  m_radiusXYmax = 3700;
    G4double  m_radiusXYmin = 1500;
    G4double  m_Zmax = 1000;
    G4int counterON = 0;
    G4int counterOFF = 0;
    G4int counterNothingOutside = 0;
    G4int counterNothingInside = 0;
    G4int counterWTF = 0;
    G4int counterFieldExist = 0;
    G4int counterFieldNotExist = 0;
};
#endif
